package com.sun.xml.internal.bind.v2.runtime.reflect.opt;

import java.security.AccessController;
import java.security.PrivilegedAction;
import com.sun.xml.internal.bind.Util;
import java.util.WeakHashMap;
import java.lang.reflect.InvocationTargetException;
import com.sun.xml.internal.bind.v2.runtime.reflect.Accessor;
import java.util.HashMap;
import java.util.logging.Level;
import java.lang.reflect.Method;
import java.util.logging.Logger;
import java.lang.ref.WeakReference;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

final class Injector
{
    private static final ReentrantReadWriteLock irwl;
    private static final Lock ir;
    private static final Lock iw;
    private static final Map<ClassLoader, WeakReference<Injector>> injectors;
    private static final Logger logger;
    private final Map<String, Class> classes;
    private final ReentrantReadWriteLock rwl;
    private final Lock r;
    private final Lock w;
    private final ClassLoader parent;
    private final boolean loadable;
    private static final Method defineClass;
    private static final Method resolveClass;
    private static final Method findLoadedClass;
    
    static Class inject(ClassLoader cl, String className, byte[] image) {
        Injector injector = get(cl);
        if (injector != null) {
            return injector.inject(className, image);
        }
        return null;
    }
    
    static Class find(ClassLoader cl, String className) {
        Injector injector = get(cl);
        if (injector != null) {
            return injector.find(className);
        }
        return null;
    }
    
    private static Injector get(ClassLoader cl) {
        Injector injector = null;
        Injector.ir.lock();
        WeakReference<Injector> wr;
        try {
            wr = (WeakReference)Injector.injectors.get(cl);
        }
        finally {
            Injector.ir.unlock();
        }
        if (wr != null) {
            injector = (Injector)wr.get();
        }
        if (injector == null) {
            try {
                wr = new WeakReference(injector = new Injector(cl));
                Injector.iw.lock();
                try {
                    if (!Injector.injectors.containsKey(cl)) {
                        Injector.injectors.put(cl, wr);
                    }
                }
                finally {
                    Injector.iw.unlock();
                }
            }
            catch (SecurityException e) {
                Injector.logger.log(Level.FINE, "Unable to set up a back-door for the injector", e);
                return null;
            }
        }
        return injector;
    }
    
    private Injector(ClassLoader parent) {
        this.classes = new HashMap();
        this.rwl = new ReentrantReadWriteLock();
        this.r = this.rwl.readLock();
        this.w = this.rwl.writeLock();
        this.parent = parent;
        assert parent != null;
        boolean loadableCheck = false;
        try {
            loadableCheck = (parent.loadClass(Accessor.class.getName()) == Accessor.class);
        }
        catch (ClassNotFoundException ex) {}
        this.loadable = loadableCheck;
        return;
    }
    
    private Class inject(String className, byte[] image) {
        if (!this.loadable) {
            return null;
        }
        boolean wlocked = false;
        boolean rlocked = false;
        try {
            this.r.lock();
            rlocked = true;
            Class<Object> c = (Class)this.classes.get(className);
            this.r.unlock();
            rlocked = false;
            if (c == null) {
                try {
                    c = (Class)Injector.findLoadedClass.invoke(this.parent, new Object[] { className.replace('/', '.') });
                }
                catch (IllegalArgumentException e) {
                    Injector.logger.log(Level.FINE, "Unable to find " + className, e);
                }
                catch (IllegalAccessException e2) {
                    Injector.logger.log(Level.FINE, "Unable to find " + className, e2);
                }
                catch (InvocationTargetException e3) {
                    Throwable t = e3.getTargetException();
                    Injector.logger.log(Level.FINE, "Unable to find " + className, t);
                }
                if (c != null) {
                    this.w.lock();
                    wlocked = true;
                    this.classes.put(className, c);
                    this.w.unlock();
                    wlocked = false;
                }
            }
            if (c == null) {
                this.r.lock();
                rlocked = true;
                c = (Class)this.classes.get(className);
                this.r.unlock();
                rlocked = false;
                if (c == null) {
                    try {
                        c = (Class)Injector.defineClass.invoke(this.parent, new Object[] { className.replace('/', '.'), image, Integer.valueOf(0), Integer.valueOf(image.length) });
                        Injector.resolveClass.invoke(this.parent, new Object[] { c });
                    }
                    catch (IllegalAccessException e2) {
                        Injector.logger.log(Level.FINE, "Unable to inject " + className, e2);
                        __Null _Null = null;
                    }
                    catch (InvocationTargetException e3) {
                        Throwable t = e3.getTargetException();
                        if ((t instanceof LinkageError)) {
                            Injector.logger.log(Level.FINE, "duplicate class definition bug occured? Please report this : " + className, t);
                        }
                        else {
                            Injector.logger.log(Level.FINE, "Unable to inject " + className, t);
                        }
                        __Null _Null2 = null;
                    }
                    catch (SecurityException e4) {
                        Injector.logger.log(Level.FINE, "Unable to inject " + className, e4);
                        __Null _Null3 = null;
                    }
                    catch (LinkageError e5) {
                        Injector.logger.log(Level.FINE, "Unable to inject " + className, e5);
                        __Null _Null4 = null;
                    }
                    this.w.lock();
                    wlocked = true;
                    if (!this.classes.containsKey(className)) {
                        this.classes.put(className, c);
                    }
                    this.w.unlock();
                    wlocked = false;
                }
            }
        }
        finally {
            if (rlocked) {
                this.r.unlock();
            }
            if (wlocked) {
                this.w.unlock();
            }
        }
    }
    
    private Class find(String className) {
        this.r.lock();
        try {
            return (Class)this.classes.get(className);
        }
        finally {
            this.r.unlock();
        }
    }
    
    static {
        irwl = new ReentrantReadWriteLock();
        ir = Injector.irwl.readLock();
        iw = Injector.irwl.writeLock();
        injectors = new WeakHashMap();
        logger = Util.getClassLogger();
        try {
            defineClass = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] { String.class, byte[].class, Integer.TYPE, Integer.TYPE });
            resolveClass = ClassLoader.class.getDeclaredMethod("resolveClass", new Class[] { Class.class });
            findLoadedClass = ClassLoader.class.getDeclaredMethod("findLoadedClass", new Class[] { String.class });
        }
        catch (NoSuchMethodException e) {
            throw new NoSuchMethodError(e.getMessage());
        }
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Injector.defineClass.setAccessible(true);
                Injector.resolveClass.setAccessible(true);
                Injector.findLoadedClass.setAccessible(true);
                return null;
            }
        });
    }
}
